@@ -11,8 +11,8 @@ class AutomaticMapper:
 
 
 class ManualMapper:
-    def __init__(self):
-        self.rules = []  # a rule is a function
+    def __init__(self, rules=[]):
+        self.rules = rules  # a rule is a function
         self.urls = {}
 
     def map(self, site):
@@ -38,18 +38,52 @@ class DefaultFeedMapper:
         }
 
 
-class DefaultPageMapper:
-    def __init__(self):
+class DefaultCollectionMapper:
+    def __init__(self, collection_name, base_url="", template="collection_item.html"):
         self.urls = {}
+        self.default_template = template
+        self.base_url = base_url
+        self.collection_name = collection_name
+
+    def map(self, site):
+        urls = {}
+        collection = site[self.collection_name]
+        for object in collection:
+            urls.update(
+                {
+                    f"{self.collection_name}.{object['slug']}": {
+                        "url": self.base_url
+                        + f"/{self.collection_name}/{object['slug']}/",
+                        "data": object,
+                        "default_template": self.default_template,
+                        "renderer": "main_renderer",
+                    }
+                }
+            )
+
+        return urls
+
+
+class DefaultPageMapper:
+    def __init__(self, template="page.html"):
+        self.urls = {}
+        self.default_template = template
 
     def map(self, site):
         for slug in site["pages"]:
+            page = site["pages"][slug]
+            try:
+                url = page["url"]
+            except:
+                url = f"/{slug}/"
+                if slug == "index":
+                    url = "/"
             self.urls.update(
                 {
                     slug: {
-                        "url": f"/{slug}/",
-                        "data": site["pages"][slug],
-                        "default_template": "page.html",
+                        "url": url,
+                        "data": page,
+                        "default_template": self.default_template,
                         "renderer": "main_renderer",
                     }
                 }
@@ -63,28 +97,16 @@ class DefaultBlogMapper:
         self.base_url = base_url
 
     def map(self, site):
-        self.urls.update(self.posts(site))
+        self.urls.update(
+            DefaultCollectionMapper(
+                base_url=self.base_url, collection_name="posts", template="post.html"
+            ).map(site)
+        )
         self.urls.update(self.blog_index(site))
         self.urls.update(self.tags(site))
 
         return self.urls
 
-    def posts(self, site):
-        urls = {}
-        for post in site["posts"]:
-            urls.update(
-                {
-                    f"posts.{post['slug']}": {
-                        "url": self.base_url + f"/posts/{post['slug']}/",
-                        "data": post,
-                        "default_template": "post.html",
-                        "renderer": "main_renderer",
-                    }
-                }
-            )
-
-        return urls
-
     def blog_index(self, site):
         urls = {}
         posts = site["posts"][1:]
@@ -106,7 +128,7 @@ class DefaultBlogMapper:
                 previous_page_url = "/"
             else:
                 previous_page_url = ""
-            if i < len(posts):
+            if i < len(paginated_posts):
                 next_page_url = f"/index/{i+1}/"
             else:
                 next_page_url = ""
@@ -114,8 +136,10 @@ class DefaultBlogMapper:
             paginator = {
                 "posts": posts,
                 "index": i,
-                "next_page_url": self.base_url + next_page_url,
-                "previous_page_url": self.base_url + previous_page_url,
+                "next_page_url": self.base_url + next_page_url if next_page_url else "",
+                "previous_page_url": self.base_url + previous_page_url
+                if previous_page_url
+                else "",
             }
             data = {
                 "title": "blog",
@@ -159,7 +183,7 @@ class DefaultBlogMapper:
                     previous_page_url = f"/tags/{tag['slug']}/"
                 else:
                     previous_page_url = ""
-                if i < len(posts):
+                if i < len(paginated_posts):
                     next_page_url = f"/tags/{tag['slug']}/{i+1}/"
                 else:
                     next_page_url = ""
@@ -167,8 +191,12 @@ class DefaultBlogMapper:
                 paginator = {
                     "posts": posts,
                     "index": i,
-                    "next_page_url": self.base_url + next_page_url,
-                    "previous_page_url": self.base_url + previous_page_url,
+                    "next_page_url": self.base_url + next_page_url
+                    if next_page_url
+                    else "",
+                    "previous_page_url": self.base_url + previous_page_url
+                    if previous_page_url
+                    else "",
                 }
                 data = {
                     "title": f"Posts tagged {tag['name']}",
