@@ -4,16 +4,12 @@ import os
 
 
 class DefaultSiteRenderer:
-    def __init__(self, name="main_renderer", template_folder="templates"):
+    def __init__(
+        self, url_function=None, name="main_renderer", template_folder="templates"
+    ):
         self.name = name
         self.template_folder = template_folder
-
-    def url(self, name):
-        try:
-            result = self.map[name]["url"]
-        except:
-            result = self.map[name + ".1"]["url"]
-        return result
+        self.url_function = url_function
 
     def date_to_string(self, date):
         return date.strftime("%b %d, %Y")
@@ -31,8 +27,9 @@ class DefaultSiteRenderer:
                 template = page["default_template"]
             jinja_template = env.get_template(template)
             jinja_template.globals.update(
-                url=self.url, date_to_string=self.date_to_string
+                url=self.url_function, date_to_string=self.date_to_string
             )
+            page["data"]["url"] = page["url"]
             rendered_file = jinja_template.render(page=page["data"], site=site)
             os.makedirs(build_location + page["url"], exist_ok=True)
             with open(build_location + page["url"] + "index.html", "w") as f:
@@ -50,10 +47,13 @@ class DefaultFeedRenderer:
                 continue
             fg = FeedGenerator()
             fg.title(site["config"]["name"])
-            fg.id(base_url)
+            if not base_url:
+                fg.id("base_url/")
+            else:
+                fg.id(base_url)
             fg.link({"href": base_url})
             fg.link({"href": base_url + "/" + name, "rel": "self"})
-            for post in site["posts"][:10]:
+            for post in site["posts"]:
                 fe = fg.add_entry()
                 fe.title(post["title"])
                 fe.id(base_url + map["posts." + post["slug"]]["url"])
